/*
 * LiPa - LIving PAge, in-browser screensavers
 * v0.5.0, 2018.02.12
 * Copyright (c) 2018 Sunkware
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * WWW: sunkware.org				E-MAIL: sunkware@gmail.com
 *
 * This program uses html2canvas, v1.0.0-alpha.9 <https://html2canvas.hertzen.com>
 * Copyright (c) 2018 Niklas von Hertzen <https://hertzen.com>
 */


// "Namespace"

var LiPa = {};


// Primary parameters

LiPa.pathToScripts = ".";
LiPa.filepathToBackground = "./lipa_backgr.jpg";

LiPa.delay = 4000; // msec

LiPa.modules = ["barnsleyer", "blurrer", "drainer", "fireflyer", "flagwaver", "huecycler", "negator", "rotator", "waver1", "waver2", "zoomer"];


// Defines

LiPa.canvasOffStyle = 'position: fixed; top: 0px; left: 0px; z-index: -1; display: none;';
LiPa.canvasOnStyle = 'position: fixed; top: 0px; left: 0px; z-index: 32767; display: inline; cursor: none;';


// State

LiPa.state = null; // "frozen", "waitScreenshot", "waitStart", "active"

LiPa.timer = -1;

LiPa.startFunctions = [];
LiPa.startFunc = null;

LiPa.stopFunctions = [];
LiPa.stopFunc = null;

LiPa.overCanvas = null;
LiPa.overCanvasContext = null;

LiPa.drawCanvas = null;
LiPa.drawCanvasContext = null;
LiPa.drawImageData = null;

LiPa.screenshotData = null;

LiPa.backgrImage = null;
LiPa.backgrDrawImageData = null;


// Functions

LiPa.loadScript = function(filepath) {
	var script = document.createElement('script');
	script.type = 'text/javascript';
	script.src = filepath;
	document.body.appendChild(script);
}

LiPa.loadModule = function(filepath) {
	LiPa.loadScript(LiPa.pathToScripts + "/lipa_" + LiPa.modules[i] + ".js");
}

LiPa.addStartStopFunctions = function(funcStart, funcStop) {
	LiPa.startFunctions[LiPa.startFunctions.length] = funcStart;
	LiPa.stopFunctions[LiPa.stopFunctions.length] = funcStop;
}

LiPa.makeControlEntity = function() {
	return {scrData		: LiPa.screenshotData,
			backgrData  : LiPa.backgrDrawImageData,
			drawCtx		: LiPa.drawCanvasContext,
			drawData 	: LiPa.drawImageData,
			width		: LiPa.drawCanvas.width,
			height		: LiPa.drawCanvas.height};
}


LiPa.makeBackground = function() {
	var canvas = document.createElement('canvas'); // off-screen
	canvas.width = window.innerWidth;
	canvas.height = window.innerHeight;
	var context = canvas.getContext('2d');

// Rescale image so that screen includes 'maximum' of its area, outwards from center

	var scaleHor = canvas.width / LiPa.backgrImage.width;
	var scaleVer = canvas.height / LiPa.backgrImage.height;
	var scale = Math.max(scaleHor, scaleVer);

	var startX = 0.5 * canvas.width - 0.5 * LiPa.backgrImage.width * scale;
	var startY = 0.5 * canvas.height - 0.5 * LiPa.backgrImage.height * scale;

	context.drawImage(LiPa.backgrImage, startX, startY, LiPa.backgrImage.width * scale, LiPa.backgrImage.height * scale);
	LiPa.backgrDrawImageData = context.getImageData(0, 0, canvas.width, canvas.height);
}


LiPa.initCanvases = function() {
	LiPa.drawCanvas = document.createElement('canvas'); // off-screen
	LiPa.drawCanvas.width = window.innerWidth;
	LiPa.drawCanvas.height = window.innerHeight;
	LiPa.drawCanvasContext = LiPa.drawCanvas.getContext('2d');
	LiPa.drawImageData = LiPa.drawCanvasContext.getImageData(0, 0, LiPa.drawCanvas.width, LiPa.drawCanvas.height);

	LiPa.overCanvas = document.createElement('canvas');
	LiPa.overCanvas.width = window.innerWidth;
	LiPa.overCanvas.height = window.innerHeight;
	LiPa.overCanvasContext = LiPa.overCanvas.getContext('2d');

	LiPa.overCanvas.style = LiPa.canvasOffStyle;
	document.body.appendChild(LiPa.overCanvas);
}

LiPa.resizeCanvasesToWindow = function() {
	var width = window.innerWidth;
	var height = window.innerHeight;

	LiPa.overCanvas.width = width;
	LiPa.overCanvas.height = height;

	LiPa.drawCanvas.width = width;
	LiPa.drawCanvas.height = height;
	LiPa.drawImageData = LiPa.drawCanvasContext.getImageData(0, 0, LiPa.drawCanvas.width, LiPa.drawCanvas.height);
}


LiPa.showCanvas = function() {
	LiPa.overCanvasContext.putImageData(LiPa.drawCanvasContext.getImageData(0, 0, LiPa.drawCanvas.width, LiPa.drawCanvas.height), 0, 0);
}

LiPa.showDirect = function() { // faster...
	LiPa.overCanvasContext.putImageData(LiPa.drawImageData, 0, 0);
}


LiPa.start = function() {
	LiPa.state = "active";

	LiPa.overCanvasContext.putImageData(LiPa.screenshotData, 0, 0);
	LiPa.overCanvas.style = LiPa.canvasOnStyle;

	var iModule = Math.floor(Math.random() * LiPa.modules.length);

	LiPa.startFunc = LiPa.startFunctions[iModule];
	LiPa.stopFunc = LiPa.stopFunctions[iModule];

	LiPa.startFunc(LiPa.makeControlEntity());
}

LiPa.stop = function() {
	LiPa.stopFunc();
	LiPa.overCanvas.style = LiPa.canvasOffStyle;
}


// Take "screenshot" using HTML2Canvas
LiPa.getScreenshotAndWait = function() {
    html2canvas(document.body).then(function(canvas) {
        LiPa.screenshotData = canvas.getContext('2d').getImageData(window.scrollX, window.scrollY, window.innerWidth, window.innerHeight);

		LiPa.timer = window.setTimeout(LiPa.start, Math.floor(LiPa.delay / 2));
    	LiPa.state = "waitStart";
    });
}


LiPa.freeze = function() {
	if (LiPa.state == "active") {
		LiPa.stop();
	}
	if (LiPa.timer != -1) {
		window.clearTimeout(LiPa.timer);
		LiPa.timer = -1;
	}
	LiPa.state = "frozen";
}

LiPa.reset = function() {
	LiPa.freeze();
	LiPa.timer = window.setTimeout(LiPa.getScreenshotAndWait, Math.floor(LiPa.delay / 2));
	LiPa.state = "waitScreenshot";
}


LiPa.init2 = function() {
// By now, LiPa.backgrImage should be loaded
	LiPa.initCanvases();

	LiPa.makeBackground();

	LiPa.reset();
}

LiPa.init = function() {
	console.log("LiPa: loading html2canvas...");
	LiPa.loadScript(LiPa.pathToScripts + '/html2canvas.min.js');

	for (i = 0; i < LiPa.modules.length; i++) {
		console.log("LiPa: loading module '" + LiPa.modules[i] + "'...");
		LiPa.loadModule(LiPa.modules[i]);
	}

	LiPa.backgrImage = new Image();
	LiPa.backgrImage.src = LiPa.filepathToBackground;
	LiPa.backgrImage.onload = LiPa.init2;
}
    

// Listeners

document.addEventListener('mousemove', function(event) {
	LiPa.reset();
}, false);

document.addEventListener('scroll', function(event) {
	LiPa.reset();
}, false);

document.addEventListener('click', function(event) {
	LiPa.reset();
}, false);

document.addEventListener('mousedown', function(event) {
	LiPa.reset();
}, false);

document.addEventListener('mouseup', function(event) {
	LiPa.reset();
}, false);

document.addEventListener('keypress', function(event) {
	LiPa.reset();
}, false);


window.addEventListener('blur', function(event) {
	LiPa.freeze();
	console.log("LiPa: lost focus, freeze.");
}, false);

window.addEventListener('focus', function(event) {
	LiPa.reset();
	console.log("LiPa: gained focus, reset.");
}, false);


window.addEventListener('resize', function() {
	LiPa.reset();
	LiPa.resizeCanvasesToWindow();
	LiPa.makeBackground();
}, false);


window.addEventListener('load', function() {
	LiPa.init();
}, false);
