/*
 * LiPa module Barnsleyer - applying lenses to monochromed page to project its copies onto itself
 * v0.5.0, 2018.02.12
 * Copyright (c) 2018 Sunkware
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * WWW: sunkware.org				E-MAIL: sunkware@gmail.com
 */


// "Namespace"

LiPa.Barnsleyer = {};
LiPa.Barnsleyer.modname = "Barnsleyer";

// Primary parameters

LiPa.Barnsleyer.monochromingduration = 2000;

LiPa.Barnsleyer.numberMin = 8;
LiPa.Barnsleyer.numberMax = 12;
LiPa.Barnsleyer.number = null;

LiPa.Barnsleyer.shadeMin = 0.75;
LiPa.Barnsleyer.shadeMax = 0.95;
LiPa.Barnsleyer.shade = null;

LiPa.Barnsleyer.scaleMin = 0.25;
LiPa.Barnsleyer.scaleMax = 0.8;

LiPa.Barnsleyer.frateMonochroming = 30;
LiPa.Barnsleyer.frate = 4;

// State

LiPa.Barnsleyer.ctrlEnt = null;

LiPa.Barnsleyer.beginTime = 0;

LiPa.Barnsleyer.timer = -1;

LiPa.Barnsleyer.lenses = null;

LiPa.Barnsleyer.lastWidth = -1;
LiPa.Barnsleyer.lastHeight = -1;
LiPa.Barnsleyer.pixbuf = null;

// Functions

LiPa.Barnsleyer.randsign = function() {
	return (1 - 2 * Math.floor(2 * Math.random()));
}

LiPa.Barnsleyer.randrange = function(min, max) {
	return (min + (max - min) * Math.random());
}

LiPa.Barnsleyer.randintrange = function(min, max) {
	return Math.floor( min + (max - min + 1) * Math.random() );
}

LiPa.Barnsleyer.initPixbuf = function() {
	var width = LiPa.Barnsleyer.ctrlEnt.width;
	var height = LiPa.Barnsleyer.ctrlEnt.height;	

	if ( (LiPa.Barnsleyer.lastWidth != width) || (LiPa.Barnsleyer.lastHeight != height) ) {
		LiPa.Barnsleyer.pixbuf = new Array(width * height);
		LiPa.Barnsleyer.lastWidth = width;
		LiPa.Barnsleyer.lastHeight = height;
	}
}

LiPa.Barnsleyer.initLenses = function() {
	LiPa.Barnsleyer.lenses = new Array(LiPa.Barnsleyer.number);

	for (i = 0; i < LiPa.Barnsleyer.number; i++) {
		LiPa.Barnsleyer.lenses[i] = {	cx: Math.floor(LiPa.Barnsleyer.ctrlEnt.width * Math.random()),
										cy: Math.floor(LiPa.Barnsleyer.ctrlEnt.height * Math.random()),
										scale: LiPa.Barnsleyer.randrange(LiPa.Barnsleyer.scaleMin, LiPa.Barnsleyer.scaleMax),
										rotangle: LiPa.Barnsleyer.randrange(0, 2 * Math.PI),
										flip: LiPa.Barnsleyer.randsign() };
	}
}

LiPa.Barnsleyer.copyScrToDraw = function() {
	var scrData = LiPa.Barnsleyer.ctrlEnt.scrData;
	var drawData = LiPa.Barnsleyer.ctrlEnt.drawData;
	var width = LiPa.Barnsleyer.ctrlEnt.width;
	var height = LiPa.Barnsleyer.ctrlEnt.height;

	var offs = 0;

	for (y = 0; y < height; y++) {
		for (x = 0; x < width; x++) {
			for (i = 0; i < 3; i++) {
				drawData.data[offs] = scrData.data[offs];
				offs++;
			}
			drawData.data[offs] = 0xFF;
			offs++;
		}
	}
}

LiPa.Barnsleyer.makeMonoImg = function() {
	var scrData = LiPa.Barnsleyer.ctrlEnt.scrData;
	var width = LiPa.Barnsleyer.ctrlEnt.width;
	var height = LiPa.Barnsleyer.ctrlEnt.height;

	var pixbuf = LiPa.Barnsleyer.pixbuf;

	var boffs = 0, soffs = 0;
	var lum, sumlum = 0;

	for (y = 0; y < height; y++) {
		for (x = 0; x < width; x++) {
			lum = (77 * scrData.data[soffs] + 150 * scrData.data[soffs + 1] + 29 * scrData.data[soffs + 2]) >> 0x8; // from: Salomon D., Data Compression Methods, 3.7.1
			sumlum += lum;

			pixbuf[boffs] = lum;

			boffs++;
			soffs += 4;
		}
	}

	sumlum /= (width * height);
	if (sumlum < 0x80) { // negate, so "most" of image is lighter
		boffs = 0;
		for (y = 0; y < height; y++) {
			for (x = 0; x < width; x++) {
				pixbuf[boffs] = 0xFF - pixbuf[boffs];
				boffs++;
			}
		}
	}
}

LiPa.Barnsleyer.drawMonoImg = function(level) {
	var drawData = LiPa.Barnsleyer.ctrlEnt.drawData;
	var width = LiPa.Barnsleyer.ctrlEnt.width;
	var height = LiPa.Barnsleyer.ctrlEnt.height;

	var pixbuf = LiPa.Barnsleyer.pixbuf;

	level = Math.floor(0x100 * level);
	opplevel = 0x100 - level;

	var boffs = 0, doffs = 0;
	var lum;

	for (y = 0; y < height; y++) {
		for (x = 0; x < width; x++) {
			lum = pixbuf[boffs];

			drawData.data[doffs] = (level * lum + opplevel * drawData.data[doffs]) >> 0x8;
			drawData.data[doffs + 1] = (level * lum + opplevel * drawData.data[doffs + 1]) >> 0x8;
			drawData.data[doffs + 2] = (level * lum + opplevel * drawData.data[doffs + 2]) >> 0x8;

			boffs++;
			doffs += 4;
		}
	}
}

LiPa.Barnsleyer.barnsleize = function() {
	var drawData = LiPa.Barnsleyer.ctrlEnt.drawData;
	var width = LiPa.Barnsleyer.ctrlEnt.width;
	var height = LiPa.Barnsleyer.ctrlEnt.height;

	var shade = Math.floor(LiPa.Barnsleyer.shade * 0x100);	
	var pixbuf = LiPa.Barnsleyer.pixbuf;

	var boffs = 0;
	for (y = 0; y < height; y++) {
		for (x = 0; x < width; x++) {
			pixbuf[boffs] = -1; // "no pixel projected here"

			boffs++;
		}
	}

	var lense;
	var ix, iy, cx, cy, lwidth, lheight, lcx, lcy, rx, ry, px, py, sx, sy;
	var angCos, angSin;
	var lxtab, lytab;
	var doffs;

	for (i = 0; i < LiPa.Barnsleyer.number; i++) {
		lense = LiPa.Barnsleyer.lenses[i];

		cx = lense.cx;
		cy = lense.cy;

		invscale = 1.0 / lense.scale;

		lwidth = Math.floor(width * lense.scale);
		lxtab = new Array(lwidth << 1);
		for (ix = 0; ix < (lwidth << 1); ix++) {
			lxtab[ix] = Math.min(Math.floor(0.5 * ix * invscale), width - 1);
		}

		lheight = Math.floor(height * lense.scale);
		lytab = new Array(lheight << 1);
		for (iy = 0; iy < (lheight << 1); iy++) {
			lytab[iy] = Math.min(Math.floor(0.5 * iy * invscale), height - 1);
		}

		lcx = 0.5 * lwidth;
		lcy = 0.5 * lheight;

		angCos = Math.cos(lense.rotangle);
		angSin = Math.sin(lense.rotangle);

		flip = lense.flip;	

		iy = 0;
		for (y = 0; y < lheight; y += 0.5) {
			ry = y - lcy;
			py = lytab[iy];
			ix = 0;
			for (x = 0; x < lwidth; x += 0.5) {
				rx = x - lcx;
				px = lxtab[ix];
				sx = Math.floor(cx + flip * (rx * angCos - ry * angSin));
				sy = Math.floor(cy + (rx * angSin + ry * angCos));
				if ((sx >= 0) && (sy >= 0) && (sx < width) && (sy < height)) {
					boffs = sy * width + sx;
					doffs = (py * width + px) << 2;

					pixbuf[boffs] = (drawData.data[doffs] * shade) >> 0x8;
				}

				ix++;	
			}

			iy++;
		}
	}

// Replacing
	boffs = 0;
	doffs = 0;
	var lum;

	for (y = 0; y < height; y++) {
		for (x = 0; x < width; x++) {
			lum = pixbuf[boffs];
			if (lum != -1) {
				drawData.data[doffs] = lum;
				drawData.data[doffs + 1] = lum;
				drawData.data[doffs + 2] = lum;
			}

			boffs++;
			doffs += 4;
		}
	}
}

LiPa.Barnsleyer.main = function() {
	LiPa.Barnsleyer.barnsleize();

	LiPa.showDirect();

	LiPa.Barnsleyer.timer = window.setTimeout(LiPa.Barnsleyer.main, Math.floor(1000 / LiPa.Barnsleyer.frate));
}

LiPa.Barnsleyer.mainMonochroming = function() {
	var time = Date.now() - LiPa.Barnsleyer.beginTime;

	if (time < LiPa.Barnsleyer.monochromingduration) {
		var level = time / LiPa.Barnsleyer.monochromingduration;
		LiPa.Barnsleyer.drawMonoImg(level);
		LiPa.showDirect();
		LiPa.Barnsleyer.timer = window.setTimeout(LiPa.Barnsleyer.mainMonochroming, Math.floor(1000 / LiPa.Barnsleyer.frateMonochroming));
	} else {
		LiPa.Barnsleyer.drawMonoImg(1);
		LiPa.Barnsleyer.main();
	}
}

LiPa.Barnsleyer.start = function(ctrlEnt) {
	console.log(LiPa.Barnsleyer.modname + " started");

	LiPa.Barnsleyer.ctrlEnt = ctrlEnt;

	LiPa.Barnsleyer.number = LiPa.Barnsleyer.randintrange(LiPa.Barnsleyer.numberMin, LiPa.Barnsleyer.numberMax);
	LiPa.Barnsleyer.shade = LiPa.Barnsleyer.randrange(LiPa.Barnsleyer.shadeMin, LiPa.Barnsleyer.shadeMax);

	LiPa.Barnsleyer.initPixbuf();
	LiPa.Barnsleyer.initLenses();
	LiPa.Barnsleyer.makeMonoImg();
	LiPa.Barnsleyer.copyScrToDraw();
	LiPa.Barnsleyer.beginTime = Date.now();
	LiPa.Barnsleyer.mainMonochroming();
}

LiPa.Barnsleyer.stop = function() {
	if (LiPa.Barnsleyer.timer != -1) {
		window.clearTimeout(LiPa.Barnsleyer.timer);
		LiPa.Barnsleyer.timer = -1;
	}

	console.log(LiPa.Barnsleyer.modname + " stopped");
}

// Registering

LiPa.addStartStopFunctions(LiPa.Barnsleyer.start, LiPa.Barnsleyer.stop);
