/*
 * LiPa module Blurrer - blurring the page
 * v0.5.0, 2018.02.12
 * Copyright (c) 2018 Sunkware
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * WWW: sunkware.org				E-MAIL: sunkware@gmail.com
 */


// "Namespace"

LiPa.Blurrer = {};
LiPa.Blurrer.modname = "Blurrer";

// Primary parameters

LiPa.Blurrer.intensity = 0.0625;
LiPa.Blurrer.frate = 30;

// State

LiPa.Blurrer.ctrlEnt = null;

LiPa.Blurrer.beginTime = 0;

LiPa.Blurrer.timer = -1;

// Functions

LiPa.Blurrer.copyScrToDraw = function() {
	var scrData = LiPa.Blurrer.ctrlEnt.scrData;
	var drawData = LiPa.Blurrer.ctrlEnt.drawData;
	var width = LiPa.Blurrer.ctrlEnt.width;
	var height = LiPa.Blurrer.ctrlEnt.height;

	var offs = 0;

	for (y = 0; y < height; y++) {
		for (x = 0; x < width; x++) {
			for (i = 0; i < 3; i++) {
				drawData.data[offs] = scrData.data[offs];
				offs++;
			}
			drawData.data[offs] = 0xFF;
			offs++;
		}
	}
}

LiPa.Blurrer.blur = function() {
	var drawData = LiPa.Blurrer.ctrlEnt.drawData;
	var width = LiPa.Blurrer.ctrlEnt.width;
	var height = LiPa.Blurrer.ctrlEnt.height;

	var pnum = Math.floor(LiPa.Blurrer.intensity * width * height);

	var x, y, sx, sy, r, g, b, w, wsum;

	var weights = [[1, 2, 1],
				   [2, 16, 2],
				   [1, 2, 1]];

	var offs;

	for (i = 0; i < pnum; i++) {
		x = Math.floor(width * Math.random());
		y = Math.floor(height * Math.random());

		r = 0; g = 0; b = 0;
		wsum = 0;

		for (dy = -1; dy <= 1; dy++) {
			sy = y + dy;
			if ((sy >= 0) && (sy < height)) {
				for (dx = -1; dx <= 1; dx++) {
					sx = x + dx;
					if ((sx >= 0) && (sx < width)) {
						offs = (sy * width + sx) << 2;
						w = weights[1 + dy][1 + dx];
						wsum += w;
						r += drawData.data[offs] * w;
						g += drawData.data[offs + 1] * w;
						b += drawData.data[offs + 2] * w;
					}
				}	
			}
		}	

		offs = (y * width + x) << 2;

		wsum = 1.0 / wsum;

		drawData.data[offs] = Math.floor(wsum * r);
		drawData.data[offs + 1] = Math.floor(wsum * g);
		drawData.data[offs + 2] = Math.floor(wsum * b);
//		drawData.data[offs + 3] = 0xFF;
	}
}

LiPa.Blurrer.main = function() {
	LiPa.Blurrer.blur();

	LiPa.showDirect();

	LiPa.Blurrer.timer = window.setTimeout(LiPa.Blurrer.main, Math.floor(1000 / LiPa.Blurrer.frate));
}

LiPa.Blurrer.start = function(ctrlEnt) {
	console.log(LiPa.Blurrer.modname + " started");

	LiPa.Blurrer.ctrlEnt = ctrlEnt;
	LiPa.Blurrer.copyScrToDraw();
	LiPa.Blurrer.beginTime = Date.now();
	LiPa.Blurrer.main();
}

LiPa.Blurrer.stop = function() {
	if (LiPa.Blurrer.timer != -1) {
		window.clearTimeout(LiPa.Blurrer.timer);
		LiPa.Blurrer.timer = -1;
	}

	console.log(LiPa.Blurrer.modname + " stopped");
}

// Registering

LiPa.addStartStopFunctions(LiPa.Blurrer.start, LiPa.Blurrer.stop);
