/*
 * LiPa module Drainer - draining the page into the center and back
 * v0.5.0, 2018.02.12
 * Copyright (c) 2018 Sunkware
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * WWW: sunkware.org				E-MAIL: sunkware@gmail.com
 */


// "Namespace"

LiPa.Drainer = {};
LiPa.Drainer.modname = "Drainer";

// Primary parameters

LiPa.Drainer.freq = 0.125;
LiPa.Drainer.frate = 30;

// State

LiPa.Drainer.ctrlEnt = null;

LiPa.Drainer.beginTime = 0;

LiPa.Drainer.timer = -1;

LiPa.Drainer.lastWidth = -1;
LiPa.Drainer.lastHeight = -1;
LiPa.Drainer.radiusTab = null;
LiPa.Drainer.angCosTab = null;
LiPa.Drainer.angSinTab = null;

// Functions

LiPa.Drainer.calculateTabs = function(width, height) {
	if ( (LiPa.Drainer.lastWidth != width) || (LiPa.Drainer.lastHeight != height) ) { // recalculate only if dimensions changed
		LiPa.Drainer.radiusTab = [];
		LiPa.Drainer.angCosTab = [];
		LiPa.Drainer.angSinTab = [];

		var centerX = 0.5 * width;
		var centerY = 0.5 * height;

		var rx, ry, angle;

		for (y = 0; y < height; y++) {
			ry = y - centerY;
			for (x = 0; x < width; x++) {
				rx =  x - centerX;

				LiPa.Drainer.radiusTab[LiPa.Drainer.radiusTab.length] = Math.sqrt(rx * rx + ry * ry);
				angle = Math.atan2(ry, rx);
				LiPa.Drainer.angCosTab[LiPa.Drainer.angCosTab.length] = Math.cos(angle);
				LiPa.Drainer.angSinTab[LiPa.Drainer.angSinTab.length] = Math.sin(angle);
			}
		}

		LiPa.Drainer.lastWidth = width;
		LiPa.Drainer.lastHeight = height;
	}
}

LiPa.Drainer.drawDrained = function(level) {
	var scrData = LiPa.Drainer.ctrlEnt.scrData;
	var drawData = LiPa.Drainer.ctrlEnt.drawData;
	var width = LiPa.Drainer.ctrlEnt.width;
	var height = LiPa.Drainer.ctrlEnt.height;

	var centerX = 0.5 * width;
	var centerY = 0.5 * height;

	var shift = level * Math.sqrt(centerX * centerX + centerY * centerY);

	var sx, sy, offs, doffs, soffs;

	offs = 0;
	doffs = 0;

	for (y = 0; y < height; y++) {
		for (x = 0; x < width; x++) {
			sx = Math.floor(centerX + (LiPa.Drainer.radiusTab[offs] + shift) * LiPa.Drainer.angCosTab[offs]);
			sy = Math.floor(centerY + (LiPa.Drainer.radiusTab[offs] + shift) * LiPa.Drainer.angSinTab[offs]);

			if ((sx >= 0) && (sy >= 0) && (sx < width) && (sy < height)) {
				soffs = (sy * width + sx) << 2;
				drawData.data[doffs] = (scrData.data[soffs] * 3 + drawData.data[doffs]) >> 2;
				drawData.data[doffs + 1] = (scrData.data[soffs + 1] * 3 + drawData.data[doffs + 1]) >> 2;
				drawData.data[doffs + 2] = (scrData.data[soffs + 2] * 3 + drawData.data[doffs + 2]) >> 2;
			} else {
				drawData.data[doffs] = (drawData.data[doffs] * 3) >> 2;
				drawData.data[doffs + 1] = (drawData.data[doffs + 1] * 3) >> 2;
				drawData.data[doffs + 2] = (drawData.data[doffs + 2] * 3) >> 2;
			}
			drawData.data[doffs + 3] = 0xFF;

			offs++;
			doffs += 4;
		}
	}
}

LiPa.Drainer.main = function() {
	var level = 0.5 * (1.0 - Math.cos(2.0 * Math.PI * LiPa.Drainer.freq * (Date.now() - LiPa.Drainer.beginTime) / 1000));
	LiPa.Drainer.drawDrained(level);

	LiPa.showDirect();

	LiPa.Drainer.timer = window.setTimeout(LiPa.Drainer.main, Math.floor(1000 / LiPa.Drainer.frate));
}

LiPa.Drainer.start = function(ctrlEnt) {
	console.log(LiPa.Drainer.modname + " started");

	LiPa.Drainer.ctrlEnt = ctrlEnt;
	LiPa.Drainer.calculateTabs(ctrlEnt.width, ctrlEnt.height);
	LiPa.Drainer.beginTime = Date.now();
	LiPa.Drainer.main();
}

LiPa.Drainer.stop = function() {
	if (LiPa.Drainer.timer != -1) {
		window.clearTimeout(LiPa.Drainer.timer);
		LiPa.Drainer.timer = -1;
	}

	console.log(LiPa.Drainer.modname + " stopped");
}

// Registering

LiPa.addStartStopFunctions(LiPa.Drainer.start, LiPa.Drainer.stop);
