/*
 * LiPa module Fireflyer - moving yellow circles from bottom to top
 * v0.5.0, 2018.02.12
 * Copyright (c) 2018 Sunkware
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * WWW: sunkware.org				E-MAIL: sunkware@gmail.com
 */


// "Namespace"

LiPa.Fireflyer = {};
LiPa.Fireflyer.modname = "Fireflyer";

// Primary parameters

LiPa.Fireflyer.shaduration = 2000;
LiPa.Fireflyer.shadefinal = 0.5;

LiPa.Fireflyer.number = 128;
LiPa.Fireflyer.radiusMin = 16;
LiPa.Fireflyer.radiusMax = 48;
LiPa.Fireflyer.speedMin = 0.05; // in screenheights-per-sec
LiPa.Fireflyer.speedMax = 0.1; // in screenheights-per-sec
LiPa.Fireflyer.horizAmplMin = 10;
LiPa.Fireflyer.horizAmplMax = 100;
LiPa.Fireflyer.horizFreqMin = 0.0625;
LiPa.Fireflyer.horizFreqMax = 0.125;

LiPa.Fireflyer.frate = 30;

// State

LiPa.Fireflyer.ctrlEnt = null;

LiPa.Fireflyer.beginTime = 0;

LiPa.Fireflyer.timer = -1;

LiPa.Fireflyer.fireflies = null;

// Functions

LiPa.Fireflyer.randsign = function() {
	return (1 - 2 * Math.floor(2 * Math.random()));
}

LiPa.Fireflyer.randrange = function(min, max) {
	return (min + (max - min) * Math.random());
}

LiPa.Fireflyer.drawShaded = function(level) {
	var scrData = LiPa.Fireflyer.ctrlEnt.scrData;
	var drawData = LiPa.Fireflyer.ctrlEnt.drawData;
	var width = LiPa.Fireflyer.ctrlEnt.width;
	var height = LiPa.Fireflyer.ctrlEnt.height;

	level = Math.floor(0x100 * level);

	var offs = 0;

	for (y = 0; y < height; y++) {
		for (x = 0; x < width; x++) {
			drawData.data[offs] = (scrData.data[offs] * level) >> 0x8;
			drawData.data[offs + 1] = (scrData.data[offs + 1] * level) >> 0x8;
			drawData.data[offs + 2] = (scrData.data[offs + 2] * level) >> 0x8;
			drawData.data[offs + 3] = 0xFF;
			offs += 4;
		}
	}
}

LiPa.Fireflyer.initFireflies = function() {
	LiPa.Fireflyer.fireflies = new Array(LiPa.Fireflyer.number);

	for (i = 0; i < LiPa.Fireflyer.number; i++) {
		LiPa.Fireflyer.fireflies[i] = {	radius: Math.floor(LiPa.Fireflyer.randrange(LiPa.Fireflyer.radiusMin, LiPa.Fireflyer.radiusMax)),
										speed: LiPa.Fireflyer.randrange(LiPa.Fireflyer.speedMin, LiPa.Fireflyer.speedMax),
										x: LiPa.Fireflyer.ctrlEnt.width * Math.random(),
										horizAmpl: LiPa.Fireflyer.randrange(LiPa.Fireflyer.horizAmplMin, LiPa.Fireflyer.horizAmplMax),
										horizFreq: LiPa.Fireflyer.randrange(LiPa.Fireflyer.horizFreqMin, LiPa.Fireflyer.horizFreqMax),
										horizDir: LiPa.Fireflyer.randsign(),
										lightmultab: null };

		var radius = LiPa.Fireflyer.fireflies[i].radius;
		var radius2 = radius * radius;

		LiPa.Fireflyer.fireflies[i].lightmultab = new Array(1 + radius2);
		for (j = 0; j <= radius2; j++) {
			var x = (radius - Math.sqrt(j)) / radius;
			var y = (3 - 2 * x) * x * x; // "cubic spline"
			LiPa.Fireflyer.fireflies[i].lightmultab[j] = Math.floor(0x100 * y);
		}
	}
}

LiPa.Fireflyer.drawFireflies = function(time) {
	LiPa.Fireflyer.drawShaded(LiPa.Fireflyer.shadefinal);

	var scrData = LiPa.Fireflyer.ctrlEnt.scrData;
	var drawData = LiPa.Fireflyer.ctrlEnt.drawData;
	var width = LiPa.Fireflyer.ctrlEnt.width;
	var height = LiPa.Fireflyer.ctrlEnt.height;

	var fx, fy, x, y, dist2;
	var radius, radius2, speed;
	var horizAmpl, horizFreq, horizDir, lightmultab;
	var offs;
	var multF, multD;

	for (i = 0; i < LiPa.Fireflyer.number; i++) {
		radius = LiPa.Fireflyer.fireflies[i].radius;
		radius2 = radius * radius;
		speed = LiPa.Fireflyer.fireflies[i].speed;
		horizAmpl = LiPa.Fireflyer.fireflies[i].horizAmpl;
		horizFreq = LiPa.Fireflyer.fireflies[i].horizFreq;
		horizDir = LiPa.Fireflyer.fireflies[i].horizDir;
		lightmultab = LiPa.Fireflyer.fireflies[i].lightmultab;

		fx = horizDir * horizAmpl * Math.sin(2.0 * Math.PI * horizFreq * (time / 1000));
		fx = LiPa.Fireflyer.fireflies[i].x + fx;

		fy = (speed * (time / 1000) * height) % (height + 2 * LiPa.Fireflyer.radiusMax); // looped
		fy = (height + LiPa.Fireflyer.radiusMax) - fy;

		fx = Math.floor(fx);
		fy = Math.floor(fy);

		for (dy = -radius; dy <= radius; dy++) {
			y = fy + dy;
			if ((y >= 0) && (y < height)) {
				for (dx = -radius; dx <= radius; dx++) {
					dist2 = dx * dx + dy * dy;
					if (dist2 <= radius2) {
						x = fx + dx;
						if ((x >= 0) && (x < width)) {
							offs = (y * width + x) << 2;
							multF = lightmultab[dist2];
							multD = 0x100 - multF;
							drawData.data[offs] = (multF * 0xFF + multD * drawData.data[offs]) >> 0x8;
							drawData.data[offs + 1] = (multF * 0xFF + multD * drawData.data[offs + 1]) >> 0x8;
//							drawData.data[offs + 2] = (multF * (drawData.data[offs + 2] << 1) + multD * drawData.data[offs + 2]) >> 0x8;
//							drawData.data[offs + 3] = 0xFF;
						}
					}
				}
			}
		}
	}

}

LiPa.Fireflyer.main = function() {
	var time = Date.now() - LiPa.Fireflyer.beginTime;

	LiPa.Fireflyer.drawFireflies(time);

	LiPa.showDirect();

	LiPa.Fireflyer.timer = window.setTimeout(LiPa.Fireflyer.main, Math.floor(1000 / LiPa.Fireflyer.frate));
}

LiPa.Fireflyer.mainShading = function() {
	var time = Date.now() - LiPa.Fireflyer.beginTime;

	if (time < LiPa.Fireflyer.shaduration) {
		var level = 1.0 - (1.0 - LiPa.Fireflyer.shadefinal) * time / LiPa.Fireflyer.shaduration;
		LiPa.Fireflyer.drawShaded(level);
		LiPa.showDirect();
		LiPa.Fireflyer.timer = window.setTimeout(LiPa.Fireflyer.mainShading, Math.floor(1000 / LiPa.Fireflyer.frate));
	} else {
		LiPa.Fireflyer.beginTime = Date.now();
		LiPa.Fireflyer.main();
	}
}

LiPa.Fireflyer.start = function(ctrlEnt) {
	console.log(LiPa.Fireflyer.modname + " started");

	LiPa.Fireflyer.ctrlEnt = ctrlEnt;
	LiPa.Fireflyer.initFireflies();
	LiPa.Fireflyer.beginTime = Date.now();
	LiPa.Fireflyer.mainShading();
}

LiPa.Fireflyer.stop = function() {
	if (LiPa.Fireflyer.timer != -1) {
		window.clearTimeout(LiPa.Fireflyer.timer);
		LiPa.Fireflyer.timer = -1;
	}

	console.log(LiPa.Fireflyer.modname + " stopped");
}

// Registering

LiPa.addStartStopFunctions(LiPa.Fireflyer.start, LiPa.Fireflyer.stop);
