/*
 * LiPa module Flagwaver - periodical distortions of page
 * v0.5.0, 2018.02.12
 * Copyright (c) 2018 Sunkware
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * WWW: sunkware.org				E-MAIL: sunkware@gmail.com
 */


// "Namespace"

LiPa.Flagwaver = {};
LiPa.Flagwaver.modname = "Flagwaver";

// Primary parameters

LiPa.Flagwaver.opacityFreq = 0.1;
LiPa.Flagwaver.opacityMax = 0.75;

LiPa.Flagwaver.horPeriodVerLen = 300;
LiPa.Flagwaver.horAmplitude = 31;
LiPa.Flagwaver.horFreq = 0.7153725;

LiPa.Flagwaver.verPeriodHorLen = 381;
LiPa.Flagwaver.verAmplitude = 59;
LiPa.Flagwaver.verFreq = 1.1328459;

LiPa.Flagwaver.frate = 30;

// State

LiPa.Flagwaver.ctrlEnt = null;

LiPa.Flagwaver.beginTime = 0;

LiPa.Flagwaver.timer = -1;

LiPa.Flagwaver.horShifts = null;
LiPa.Flagwaver.verShifts = null;
LiPa.Flagwaver.lastWidth = -1;
LiPa.Flagwaver.lastHeight = -1;

// Functions

LiPa.Flagwaver.initShiftabs = function() {
	var width = LiPa.Flagwaver.ctrlEnt.width;
	var height = LiPa.Flagwaver.ctrlEnt.height;

	if ((LiPa.Flagwaver.lastWidth != width) || (LiPa.Flagwaver.lastHeight != height)) {
		LiPa.Flagwaver.horShifts = new Array(height);
		LiPa.Flagwaver.verShifts = new Array(width);

		LiPa.Flagwaver.lastWidth = width;
		LiPa.Flagwaver.lastHeight = height;
	}
}

LiPa.Flagwaver.drawWaved = function(timesec) {
	var scrData = LiPa.Flagwaver.ctrlEnt.scrData;
	var backgrData = LiPa.Flagwaver.ctrlEnt.backgrData;
	var drawData = LiPa.Flagwaver.ctrlEnt.drawData;
	var width = LiPa.Flagwaver.ctrlEnt.width;
	var height = LiPa.Flagwaver.ctrlEnt.height;

	var horShifts = LiPa.Flagwaver.horShifts;
	var verShifts = LiPa.Flagwaver.verShifts;

	var phase;

	for (y = 0; y < height; y++) {
		phase = 2.0 * Math.PI * ((y % LiPa.Flagwaver.horPeriodVerLen) / LiPa.Flagwaver.horPeriodVerLen);
		horShifts[y] = LiPa.Flagwaver.horAmplitude * Math.sin(2.0 * Math.PI * LiPa.Flagwaver.horFreq * timesec + phase);
	}

	for (x = 0; x < width; x++) {
		phase = 2.0 * Math.PI * ((x % LiPa.Flagwaver.verPeriodHorLen) / LiPa.Flagwaver.verPeriodHorLen);
		verShifts[x] = LiPa.Flagwaver.verAmplitude * Math.sin(2.0 * Math.PI * LiPa.Flagwaver.verFreq * timesec + phase);
	}

	var wx, wy;
	var doffs = 0, woffs;
	var mult, multstep = 1.0 / width;

	var opacity = Math.floor(LiPa.Flagwaver.opacityMax * 0.5 * (1.0 + Math.cos(2.0 * Math.PI * LiPa.Flagwaver.opacityFreq * timesec)) * 0x100);
	var opacityOpp = 0x100 - opacity;

	for (y = 0; y < height; y++) {
		mult = 0.0;
		for (x = 0; x < width; x++) {
			wx = Math.floor(x + mult * horShifts[y]);
			wy = Math.floor(y + mult * verShifts[x]);
			if ( (wx >= 0) && (wy >= 0) && (wx < width) && (wy < height) ) {
				woffs = (wy * width + wx) << 2;
				drawData.data[doffs] = (opacity * scrData.data[woffs] + opacityOpp * backgrData.data[doffs]) >> 8;
				drawData.data[doffs + 1] = (opacity * scrData.data[woffs + 1] + opacityOpp * backgrData.data[doffs + 1]) >> 8;
				drawData.data[doffs + 2] = (opacity * scrData.data[woffs + 2] + opacityOpp * backgrData.data[doffs + 2]) >> 8;
			} else {
				drawData.data[doffs] = backgrData.data[doffs];
				drawData.data[doffs + 1] = backgrData.data[doffs + 1];
				drawData.data[doffs + 2] = backgrData.data[doffs + 2];
			}
			drawData.data[doffs + 3] = 0xFF;

			mult += multstep;
			doffs += 4;
		}
	}
}

LiPa.Flagwaver.main = function() {
	LiPa.Flagwaver.drawWaved((Date.now() - LiPa.Flagwaver.beginTime) / 1000);

	LiPa.showDirect();

	LiPa.Flagwaver.timer = window.setTimeout(LiPa.Flagwaver.main, Math.floor(1000 / LiPa.Flagwaver.frate));
}

LiPa.Flagwaver.start = function(ctrlEnt) {
	console.log(LiPa.Flagwaver.modname + " started");

	LiPa.Flagwaver.ctrlEnt = ctrlEnt;
	LiPa.Flagwaver.initShiftabs();
	LiPa.Flagwaver.beginTime = Date.now();
	LiPa.Flagwaver.main();
}

LiPa.Flagwaver.stop = function() {
	if (LiPa.Flagwaver.timer != -1) {
		window.clearTimeout(LiPa.Flagwaver.timer);
		LiPa.Flagwaver.timer = -1;
	}

	console.log(LiPa.Flagwaver.modname + " stopped");
}

// Registering

LiPa.addStartStopFunctions(LiPa.Flagwaver.start, LiPa.Flagwaver.stop);
