/*
 * LiPa module Negator - turning the page to negative and back
 * v0.5.0, 2018.02.12
 * Copyright (c) 2018 Sunkware
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * WWW: sunkware.org				E-MAIL: sunkware@gmail.com
 */


// "Namespace"

LiPa.Negator = {};
LiPa.Negator.modname = "Negator";

// Primary parameters

LiPa.Negator.freq = 0.125;
LiPa.Negator.frate = 30;

// State

LiPa.Negator.ctrlEnt = null;

LiPa.Negator.beginTime = 0;

LiPa.Negator.timer = -1;

// Functions

LiPa.Negator.drawNegated = function(level) {
	var scrData = LiPa.Negator.ctrlEnt.scrData;
	var drawData = LiPa.Negator.ctrlEnt.drawData;
	var width = LiPa.Negator.ctrlEnt.width;
	var height = LiPa.Negator.ctrlEnt.height;

	level = Math.min(level, 1.0)
	level = Math.floor(level * 0x10000);

	var size = width * height;

	var offset = 0;

	for (i = 0; i < size; i++) {
		for (j = 0; j < 3; j++) {
			drawData.data[offset] = (level * (255 - scrData.data[offset]) + (0x10000 - level) * scrData.data[offset]) >> 0x10;
			offset++;
		}
		drawData.data[offset] = 0xff; // alpha
		offset++;
	}
}

LiPa.Negator.main = function() {
	var level = 0.5 * (1.0 - Math.cos(2.0 * Math.PI * LiPa.Negator.freq * (Date.now() - LiPa.Negator.beginTime) / 1000));
	LiPa.Negator.drawNegated(level);

	LiPa.showDirect();

	LiPa.Negator.timer = window.setTimeout(LiPa.Negator.main, Math.floor(1000 / LiPa.Negator.frate));
}

LiPa.Negator.start = function(ctrlEnt) {
	console.log(LiPa.Negator.modname + " started");

	LiPa.Negator.ctrlEnt = ctrlEnt;
	LiPa.Negator.beginTime = Date.now();
	LiPa.Negator.main();
}

LiPa.Negator.stop = function() {
	if (LiPa.Negator.timer != -1) {
		window.clearTimeout(LiPa.Negator.timer);
		LiPa.Negator.timer = -1;
	}

	console.log(LiPa.Negator.modname + " stopped");
}

// Registering

LiPa.addStartStopFunctions(LiPa.Negator.start, LiPa.Negator.stop);
