/*
 * LiPa module Rotator - rotating the page around the center
 * v0.5.0, 2018.02.12
 * Copyright (c) 2018 Sunkware
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * WWW: sunkware.org				E-MAIL: sunkware@gmail.com
 */


// "Namespace"

LiPa.Rotator = {};
LiPa.Rotator.modname = "Rotator";

// Primary parameters

LiPa.Rotator.freq = 0.125;
LiPa.Rotator.frate = 30;

// State

LiPa.Rotator.ctrlEnt = null;

LiPa.Rotator.beginTime = 0;
LiPa.Rotator.direction = 0;

LiPa.Rotator.timer = -1;

// Functions

LiPa.Rotator.drawRotated = function(angle) {
	
	var scrData = LiPa.Rotator.ctrlEnt.scrData;
	var drawData = LiPa.Rotator.ctrlEnt.drawData;
	var width = LiPa.Rotator.ctrlEnt.width;
	var height = LiPa.Rotator.ctrlEnt.height;

	var centerX = 0.5 * width;
	var centerY = 0.5 * height;

	var angCos = Math.cos(angle);
	var angSin = Math.sin(angle);

	var rx, ry, nx, ny, roffs, noffs;

	roffs = 0;

	for (y = 0; y < height; y++) {
		ry = y - centerY;
		for (x = 0; x < width; x++) {
			rx = x - centerX;

			nx = Math.floor(centerX + rx * angCos - ry * angSin);
			ny = Math.floor(centerY + rx * angSin + ry * angCos);

			if ((nx >= 0) && (ny >= 0) && (nx < width) && (ny < height)) {
				noffs = (ny * width + nx) << 2;
				drawData.data[roffs] = scrData.data[noffs];
				drawData.data[roffs + 1] = scrData.data[noffs + 1];
				drawData.data[roffs + 2] = scrData.data[noffs + 2];
			} else {
				drawData.data[roffs] = 0;
				drawData.data[roffs + 1] = 0;
				drawData.data[roffs + 2] = 0;
			}
			drawData.data[roffs + 3] = 0xFF;

			roffs += 4;
		}
	}
}

LiPa.Rotator.main = function() {
	LiPa.Rotator.drawRotated( LiPa.Rotator.direction * 2.0 * Math.PI * LiPa.Rotator.freq * (Date.now() - LiPa.Rotator.beginTime) / 1000.0 );

	LiPa.showDirect();

	LiPa.Rotator.timer = window.setTimeout(LiPa.Rotator.main, Math.floor(1000 / LiPa.Rotator.frate));
}

LiPa.Rotator.start = function(ctrlEnt) {
	console.log(LiPa.Rotator.modname + " started");

	LiPa.Rotator.ctrlEnt = ctrlEnt;
	LiPa.Rotator.direction = 1 - 2 * Math.floor(2.0 * Math.random());
	LiPa.Rotator.beginTime = Date.now();
	LiPa.Rotator.main();
}

LiPa.Rotator.stop = function() {
	if (LiPa.Rotator.timer != -1) {
		window.clearTimeout(LiPa.Rotator.timer);
		LiPa.Rotator.timer = -1;
	}

	console.log(LiPa.Rotator.modname + " stopped");
}

// Registering

LiPa.addStartStopFunctions(LiPa.Rotator.start, LiPa.Rotator.stop);
