/*
 * LiPa module Waver(1) - periodical horizontal & vertical distortions of page
 * v0.5.0, 2018.02.12
 * Copyright (c) 2018 Sunkware
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * WWW: sunkware.org				E-MAIL: sunkware@gmail.com
 */


// "Namespace"

LiPa.Waver1 = {};
LiPa.Waver1.modname = "Waver(1)";

// Primary parameters

LiPa.Waver1.horPeriodVerLen = 100;
LiPa.Waver1.horAmplitude = 6;
LiPa.Waver1.horFreq = 0.5153725;

LiPa.Waver1.verPeriodHorLen = 81;
LiPa.Waver1.verAmplitude = 5;
LiPa.Waver1.verFreq = 0.9328459;

LiPa.Waver1.frate = 30;

// State

LiPa.Waver1.ctrlEnt = null;

LiPa.Waver1.beginTime = 0;

LiPa.Waver1.timer = -1;

LiPa.Waver1.horShifts = null;
LiPa.Waver1.verShifts = null;
LiPa.Waver1.lastWidth = -1;
LiPa.Waver1.lastHeight = -1;

// Functions

LiPa.Waver1.initShiftabs = function() {
	var width = LiPa.Waver1.ctrlEnt.width;
	var height = LiPa.Waver1.ctrlEnt.height;

	if ((LiPa.Waver1.lastWidth != width) || (LiPa.Waver1.lastHeight != height)) {
		LiPa.Waver1.horShifts = new Array(height);
		LiPa.Waver1.verShifts = new Array(width);

		LiPa.Waver1.lastWidth = width;
		LiPa.Waver1.lastHeight = height;
	}
}

LiPa.Waver1.drawWaved = function(timesec) {
	var scrData = LiPa.Waver1.ctrlEnt.scrData;
	var drawData = LiPa.Waver1.ctrlEnt.drawData;
	var width = LiPa.Waver1.ctrlEnt.width;
	var height = LiPa.Waver1.ctrlEnt.height;

	var horShifts = LiPa.Waver1.horShifts;
	var verShifts = LiPa.Waver1.verShifts;

	var phase;

	for (y = 0; y < height; y++) {
		phase = 2.0 * Math.PI * ((y % LiPa.Waver1.horPeriodVerLen) / LiPa.Waver1.horPeriodVerLen);
		horShifts[y] = Math.floor(LiPa.Waver1.horAmplitude * Math.sin(2.0 * Math.PI * LiPa.Waver1.horFreq * timesec + phase));
	}

	for (x = 0; x < width; x++) {
		phase = 2.0 * Math.PI * ((x % LiPa.Waver1.verPeriodHorLen) / LiPa.Waver1.verPeriodHorLen);
		verShifts[x] = Math.floor(LiPa.Waver1.verAmplitude * Math.sin(2.0 * Math.PI * LiPa.Waver1.verFreq * timesec + phase));
	}

	var wx, wy;
	var doffs = 0, woffs;

	for (y = 0; y < height; y++) {
		for (x = 0; x < width; x++) {
			wx = x + horShifts[y];
			wy = y + verShifts[x];
			if ((wx >= 0) && (wy >= 0) && (wx < width) && (wy < height)) {
				woffs = (wy * width + wx) << 2;
				drawData.data[doffs] = scrData.data[woffs];
				drawData.data[doffs + 1] = scrData.data[woffs + 1];
				drawData.data[doffs + 2] = scrData.data[woffs + 2];
			} else {
				drawData.data[doffs] = 0;
				drawData.data[doffs + 1] = 0;
				drawData.data[doffs + 2] = 0;
			}
			drawData.data[doffs + 3] = 0xFF;

			doffs += 4;
		}
	}
}

LiPa.Waver1.main = function() {
	LiPa.Waver1.drawWaved((Date.now() - LiPa.Waver1.beginTime) / 1000);

	LiPa.showDirect();

	LiPa.Waver1.timer = window.setTimeout(LiPa.Waver1.main, Math.floor(1000 / LiPa.Waver1.frate));
}

LiPa.Waver1.start = function(ctrlEnt) {
	console.log(LiPa.Waver1.modname + " started");

	LiPa.Waver1.ctrlEnt = ctrlEnt;
	LiPa.Waver1.initShiftabs();
	LiPa.Waver1.beginTime = Date.now();
	LiPa.Waver1.main();
}

LiPa.Waver1.stop = function() {
	if (LiPa.Waver1.timer != -1) {
		window.clearTimeout(LiPa.Waver1.timer);
		LiPa.Waver1.timer = -1;
	}

	console.log(LiPa.Waver1.modname + " stopped");
}

// Registering

LiPa.addStartStopFunctions(LiPa.Waver1.start, LiPa.Waver1.stop);
