/*
 * LiPa module Waver(2) - periodical radial distortions of page
 * v0.5.0, 2018.02.12
 * Copyright (c) 2018 Sunkware
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * WWW: sunkware.org				E-MAIL: sunkware@gmail.com
 */


// "Namespace"

LiPa.Waver2 = {};
LiPa.Waver2.modname = "Waver(2)";

// Primary parameters

LiPa.Waver2.freq = 1;
LiPa.Waver2.periodlen = 96;
LiPa.Waver2.amplitude = 4;

LiPa.Waver2.frate = 30;

// State

LiPa.Waver2.ctrlEnt = null;

LiPa.Waver2.beginTime = 0;

LiPa.Waver2.timer = -1;

LiPa.Waver2.lastWidth = -1;
LiPa.Waver2.lastHeight = -1;
LiPa.Waver2.radiusTab = null;
LiPa.Waver2.phaseTab = null;
LiPa.Waver2.angCosTab = null;
LiPa.Waver2.angSinTab = null;
LiPa.Waver2.sineTab = null;

// Functions

LiPa.Waver2.calculateTabs = function() {
	if (LiPa.Waver2.sineTab === null) {
		LiPa.Waver2.sineTab = new Array(0x10000);
		for (i = 0; i < 0x10000; i++) {
			LiPa.Waver2.sineTab[i] = Math.sin(2.0 * Math.PI * i / 0x10000);
		}
	}

	var width = LiPa.Waver2.ctrlEnt.width;
	var height = LiPa.Waver2.ctrlEnt.height;

	var periodlen = LiPa.Waver2.periodlen;

	if ( (LiPa.Waver2.lastWidth != width) || (LiPa.Waver2.lastHeight != height) ) { // recalculate only if dimensions changed
		LiPa.Waver2.radiusTab = [];
		LiPa.Waver2.phaseTab = [];
		LiPa.Waver2.angCosTab = [];
		LiPa.Waver2.angSinTab = [];

		var centerX = 0.5 * width;
		var centerY = 0.5 * height;

		var rx, ry, radius, angle;

		for (y = 0; y < height; y++) {
			ry = y - centerY;
			for (x = 0; x < width; x++) {
				rx =  x - centerX;

				radius = Math.sqrt(rx * rx + ry * ry);
				LiPa.Waver2.radiusTab[LiPa.Waver2.radiusTab.length] = radius;
				LiPa.Waver2.phaseTab[LiPa.Waver2.phaseTab.length] = (radius % periodlen) / periodlen;

				angle = Math.atan2(ry, rx);
				LiPa.Waver2.angCosTab[LiPa.Waver2.angCosTab.length] = Math.cos(angle);
				LiPa.Waver2.angSinTab[LiPa.Waver2.angSinTab.length] = Math.sin(angle);
			}
		}

		LiPa.Waver2.lastWidth = width;
		LiPa.Waver2.lastHeight = height;
	}
}

LiPa.Waver2.drawWaved = function(timesec) {
	var scrData = LiPa.Waver2.ctrlEnt.scrData;
	var drawData = LiPa.Waver2.ctrlEnt.drawData;
	var width = LiPa.Waver2.ctrlEnt.width;
	var height = LiPa.Waver2.ctrlEnt.height;

	var periodlen = LiPa.Waver2.periodlen;
	var amplitude = LiPa.Waver2.amplitude;
	var freq = LiPa.Waver2.freq;

	var centerX = 0.5 * width;
	var centerY = 0.5 * height;

	var sx, sy, offs, doffs, soffs;
	var radius, phase, shift;

	offs = 0;
	doffs = 0;

	for (y = 0; y < height; y++) {
		for (x = 0; x < width; x++) {
			radius = LiPa.Waver2.radiusTab[offs];
			phase = LiPa.Waver2.phaseTab[offs];
//			shift = amplitude * Math.sin(-2.0 * Math.PI * timesec * freq + phase);
			shift = amplitude * LiPa.Waver2.sineTab[Math.floor((-timesec * freq + phase) * 0x10000) & 0xFFFF]; // "-" for waves to move "out"

			radius += shift;

			sx = Math.floor(centerX + (radius + shift) * LiPa.Waver2.angCosTab[offs]);
			sy = Math.floor(centerY + (radius + shift) * LiPa.Waver2.angSinTab[offs]);

			if ((sx >= 0) && (sy >= 0) && (sx < width) && (sy < height)) {
				soffs = (sy * width + sx) << 2;
				drawData.data[doffs] = scrData.data[soffs];
				drawData.data[doffs + 1] = scrData.data[soffs + 1];
				drawData.data[doffs + 2] = scrData.data[soffs + 2];
			} else {
				drawData.data[doffs] = 0;
				drawData.data[doffs + 1] = 0;
				drawData.data[doffs + 2] = 0;
			}
			drawData.data[doffs + 3] = 0xFF;

			offs++;
			doffs += 4;
		}
	}
}

LiPa.Waver2.main = function() {
	var time = (Date.now() - LiPa.Waver2.beginTime) / 1000;
	LiPa.Waver2.drawWaved(time);

	LiPa.showDirect();

	LiPa.Waver2.timer = window.setTimeout(LiPa.Waver2.main, Math.floor(1000 / LiPa.Waver2.frate));
}

LiPa.Waver2.start = function(ctrlEnt) {
	console.log(LiPa.Waver2.modname + " started");

	LiPa.Waver2.ctrlEnt = ctrlEnt;
	LiPa.Waver2.calculateTabs();
	LiPa.Waver2.beginTime = Date.now();
	LiPa.Waver2.main();
}

LiPa.Waver2.stop = function() {
	if (LiPa.Waver2.timer != -1) {
		window.clearTimeout(LiPa.Waver2.timer);
		LiPa.Waver2.timer = -1;
	}

	console.log(LiPa.Waver2.modname + " stopped");
}

// Registering

LiPa.addStartStopFunctions(LiPa.Waver2.start, LiPa.Waver2.stop);
