/*
 * LiPa module Zoomer - zooming the periodic tiling
 * v0.5.0, 2018.02.12
 * Copyright (c) 2018 Sunkware
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * WWW: sunkware.org				E-MAIL: sunkware@gmail.com
 */


// "Namespace"

LiPa.Zoomer = {};
LiPa.Zoomer.modname = "Zoomer";

// Primary parameters

LiPa.Zoomer.maxZoom = 32;
LiPa.Zoomer.freq = 0.0625;
LiPa.Zoomer.frate = 30;

// State

LiPa.Zoomer.ctrlEnt = null;

LiPa.Zoomer.beginTime = 0;

LiPa.Zoomer.timer = -1;

// Functions

LiPa.Zoomer.mod = function(a, b) { // a, b are integer and b > 0
	var smod = a % b;
	if (smod < 0) {
		smod += b;
	}
	return smod;
}

LiPa.Zoomer.drawZoomed = function(zoom) {
	var scrData = LiPa.Zoomer.ctrlEnt.scrData;
	var drawData = LiPa.Zoomer.ctrlEnt.drawData;
	var width = LiPa.Zoomer.ctrlEnt.width;
	var height = LiPa.Zoomer.ctrlEnt.height;

	var centerX = 0.5 * width;
	var centerY = 0.5 * height;

	var rx, ry, zx, zy;

	var doffs = 0, soffs;

	for (y = 0; y < height; y++) {
		ry = y - centerY;
		for (x = 0; x < width; x++) {
			rx = x - centerX;

			zx = Math.floor(centerX + rx * zoom) % width;
			if (zx < 0) {
				zx += width;
			}

			zy = Math.floor(centerY + ry * zoom) % height;
			if (zy < 0) {
				zy += height;
			}

			soffs = (zy * width + zx) << 2;

			drawData.data[doffs] = scrData.data[soffs];
			drawData.data[doffs + 1] = scrData.data[soffs + 1];
			drawData.data[doffs + 2] = scrData.data[soffs + 2];
			drawData.data[doffs + 3] = 0xFF;

			doffs += 4;
		}
	}
}

LiPa.Zoomer.main = function() {
	var level = 0.5 * (1.0 - Math.cos(2.0 * Math.PI * LiPa.Zoomer.freq * (Date.now() - LiPa.Zoomer.beginTime) / 1000));
	var zoom = Math.pow(LiPa.Zoomer.maxZoom, level);

	LiPa.Zoomer.drawZoomed(zoom);

	LiPa.showDirect();

	LiPa.Zoomer.timer = window.setTimeout(LiPa.Zoomer.main, Math.floor(1000 / LiPa.Zoomer.frate));
}

LiPa.Zoomer.start = function(ctrlEnt) {
	console.log(LiPa.Zoomer.modname + " started");

	LiPa.Zoomer.ctrlEnt = ctrlEnt;
	LiPa.Zoomer.beginTime = Date.now();
	LiPa.Zoomer.main();	
}

LiPa.Zoomer.stop = function() {
	if (LiPa.Zoomer.timer != -1) {
		window.clearTimeout(LiPa.Zoomer.timer);
		LiPa.Zoomer.timer = -1;
	}

	console.log(LiPa.Zoomer.modname + " stopped");
}

// Registering

LiPa.addStartStopFunctions(LiPa.Zoomer.start, LiPa.Zoomer.stop);
